#pragma once

#include <stdbool.h>
#include <stdint.h>

typedef struct CircBuf {
    uint8_t writeIndex;
    uint8_t readIndex;
    uint8_t length;
    uint8_t* data;
} CircBuf;

/**
 * Set up a circular buffer struct around the given storage
 */
inline static void circbuf_init(CircBuf* buf, uint8_t* storage, uint8_t storageSize)
{
    buf->writeIndex = 0;
    buf->readIndex = 0;
    buf->length = storageSize;
    buf->data = storage;
}

inline static bool circbuf_is_empty(CircBuf* buf)
{
    return buf->writeIndex == buf->readIndex;
}

inline static uint8_t circbuf_size(CircBuf* buf)
{
    const uint8_t end = buf->readIndex;
    const uint8_t start = buf->writeIndex;
    
    if (end > start) {
        return end - start;
    } else {
        return start - end;
    }
}

/**
 * Reset the buffer's state so it reads as empty
 */
inline static void circbuf_clear(CircBuf* buf)
{
    buf->writeIndex = 0;
    buf->readIndex = 0;
}

static void circbuf_append(CircBuf* buf, uint8_t byte)
{
    uint8_t nextWriteIndex = buf->writeIndex + 1;

    if (nextWriteIndex >= buf->length) {
        nextWriteIndex = 0;
    }

    // Append the received byte if the buffer has room
    if (nextWriteIndex != buf->readIndex) {
        buf->data[buf->writeIndex] = byte;
        buf->writeIndex = nextWriteIndex;
    }
}

static uint8_t circbuf_pop(CircBuf* buf)
{
    const uint8_t value = buf->data[buf->readIndex];

    uint8_t nextReadIndex = buf->readIndex + 1;
    if (nextReadIndex >= buf->length) {
        nextReadIndex = 0;
    }

    buf->readIndex = nextReadIndex;

    return value;
}
