#include "leds.h"

#include <string.h>
#include <math.h>

// Actual value of the leds
static cRGB ledsCurrent[NUM_LEDS];

// Value the leds need to be at
// Note colour order here is BRG
static cRGB ledsTarget[NUM_LEDS] = {
    { 234, 19, 181 },
    { 255, 0, 173 },
    { 234, 19, 181 }
};

// If the target differs from the currently shown colour
// This is used to avoid running transitions when not needed
static bool ledsTargetDirty = true;

static uint8_t activeFeatures = kLedFeature_Transition | 
                                kLedFeature_Calibration |
                                kLedFeature_BrightnessLookup;

static LedDecoration decorationMode = kLedDecoration_Breathe;

void led_set_all(cRGB colour)
{
    for (int i = NUM_LEDS - 1; i >= 0; i--) {
        ledsTarget[i] = colour;
    }

    ledsTargetDirty = true;
}

void led_set_index(uint8_t index, cRGB colour)
{
    ledsTarget[index] = colour;
    ledsTargetDirty = true;
}

void led_set_feature(LedFeature feature, bool enable)
{
    if (enable) {
        activeFeatures |= feature;
    } else {
        activeFeatures &= ~feature;
    }
}

void led_set_decoration(LedDecoration mode)
{
    decorationMode = mode;
}

void led_update(void)
{
    // Copy of the current state that can be adjusted before output
    static cRGB modLeds[NUM_LEDS];

    // Fade current colours to selected colours
    if (activeFeatures & kLedFeature_Transition) {

        if (ledsTargetDirty) {
            uint8_t changed = 0;
            for (uint8_t i = 0; i < NUM_LEDS; ++i) {
                changed += cFadeTo(&ledsCurrent[i], &ledsTarget[i], 0.03);
            }

            if (!changed) {
                ledsTargetDirty = false;
            }
        }

        // Copy transition progress to temporary for output
        memcpy(modLeds, ledsCurrent, sizeof(modLeds));

    } else {

        // If not using transitions, copy straight from the target values
        memcpy(modLeds, ledsTarget, sizeof(modLeds));
    }


    if (decorationMode == kLedDecoration_Breathe) {
        static float phase;
        phase += 0.01;

        float ph1 = 1.0 + ((cos(phase) + 1.0) * 0.3);
        float ph2 = 1.0 + ((cos(phase - M_PI/6) + 1.0) * 0.2);

        cMult(&modLeds[0], ph2);
        cMult(&modLeds[1], ph1);
        cMult(&modLeds[2], ph2);
    }

    for (uint8_t i = 0; i < NUM_LEDS; ++i) {
        if (activeFeatures & kLedFeature_Calibration) {
            applyColourCalibration(
                &modLeds[i],
                CALIBRATION_RED,
                CALIBRATION_GREEN,
                CALIBRATION_BLUE
            );
        }

        if (activeFeatures & kLedFeature_BrightnessLookup) {
            applyCieBrightness(&modLeds[i]);
        }
    }

    ws2812_sendarray((uint8_t *) modLeds, NUM_LEDS * COLOR_COMPONENTS);
}
