#include <avr/io.h>
#include <avr/power.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "circbuf.h"
#include "commands.h"
#include "leds.h"

#include "vusb/usbdrv.h"

// USB data in buffer
static CircBuf _usbBuf;
static uint8_t _usbBufferStorage[64];

// State for multi-chunk transfers
static uint8_t _bytesToTransfer = 0;


usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *req = (usbRequest_t*)((void *)data);

    if (req->bmRequestType == USBRQ_TYPE_VENDOR && req->bRequest == USBRQ_HID_SET_REPORT) {
        // Store the number of expected bytes to be transferred
        _bytesToTransfer = req->wLength.bytes[0];
        return USB_NO_MSG;
    }

    return 0;
}

/**
 * Called when the host sends a chunk of data to the device
 * Up to 8 bytes are sent in each chunk
 */
uchar usbFunctionWrite(uchar *data, uchar len)
{
    for (int i = 0; i < len; i++) {
        circbuf_append(&_usbBuf, data[i]);
    }

    _bytesToTransfer -= len;

    return _bytesToTransfer <= 0;
}

static cRGB popColour()
{
    uint8_t data[3] = {0, 0, 0};

    for (uint8_t i = 0; i < 3; ++i) {
        if (!circbuf_is_empty(&_usbBuf)) {
            data[i] = circbuf_pop(&_usbBuf);
        }
    }

    return RGB(data[0], data[1], data[2]);
}

void performAction()
{
    if (_bytesToTransfer == 0) {
        if (circbuf_is_empty(&_usbBuf)) {
            // Bail out if there's nothing to read
            return;
        }

        uint8_t command = circbuf_pop(&_usbBuf);
        _bytesToTransfer = -1; // Mark transfer as picked up

        switch (command) {
            case LIGHT_CMD_SET_ALL:
                led_set_all(popColour());
                break;

            case LIGHT_CMD_SET_EACH:
                led_set_index(0, popColour());
                led_set_index(1, popColour());
                led_set_index(2, popColour());
                break;

            case LIGHT_CMD_SET_FEATURE:
                if (circbuf_size(&_usbBuf) == 2) {
                    const LedFeature feature = circbuf_pop(&_usbBuf);
                    const bool enabled = circbuf_pop(&_usbBuf);
                    led_set_feature(feature, enabled);
                }
                break;

            case LIGHT_CMD_SET_MODE:
                if (circbuf_size(&_usbBuf) == 1) {
                    led_set_decoration(
                        (LedDecoration) circbuf_pop(&_usbBuf)
                    );
                }
                break;
        }

        circbuf_clear(&_usbBuf);
    }
}

/**
 * Set up a timer to trigger led state updates at a fixed frame-rate
 */
static void setupTimer(void)
{
    // Turn off all the devices except timer0
    power_all_disable();
    power_timer1_enable();

    // Set the Timer Mode to CTC
    TCCR1B |= (1 << WGM12);

    // Set up milliseconds per frame based on clock speed
    const int FPS = 60;
    OCR1A = ((F_CPU/1024) / FPS) - 1;

    TIMSK1 |= (1 << OCIE1A);

    // Usa CLK/1024 prescaler and start the timer
    TCCR1B |= (1 << CS10) | (1 << CS12);
}

static void setupUsb() {
    cli();

    usbInit();
    usbDeviceDisconnect();

    _delay_ms(200);

    usbDeviceConnect();

    sei();
}

// Flag indicating that the next LED state should be executed
static volatile uint8_t _shouldRender = 0;

ISR_NOBLOCK
ISR (TIMER1_COMPA_vect) 
{
    _shouldRender = 1;
}

int main(void)
{
    // Recover from any watchdog timer reset mishaps
    wdt_reset();
    wdt_disable();

    // Select minimal prescaler (max system speed)
    CLKPR = 0x80;
    CLKPR = 0x00;

    // Set up buffer for use
    circbuf_init(&_usbBuf, _usbBufferStorage, sizeof(_usbBufferStorage));

    setupTimer();
    setupUsb();

    for (;;) {
        usbPoll();
        performAction();

        if (_shouldRender) {
            _shouldRender = 0;
            led_update();
        }
    }

    return 0;
}
