import sys
import time
import usb1

# Demo program to flash LEDs like an emergency vehicle's lights

VENDOR_ID = 0xf055
PRODUCT_ID = 0x05df

# Set all lights to the same colour as bytes
#
# @param { uint8_t r, g, b }
LIGHT_CMD_SET_ALL = 0xC0

# Set the colour of each LED group as a stream of bytes
#
# @param { uint8_t r, g, b, r, g, b, ... }
LIGHT_CMD_SET_EACH = 0xC1

# Load a sequence of frames into the buffer
LIGHT_CMD_PUSH_SEQUENCE = 0xC2

# Enable or disable automatic slow fade to target colour
# @param { uint8_t feature, bool enabled }
# @see LIGHT_FEATURE_*
LIGHT_CMD_SET_FEATURE = 0xCE

# Set the automatic decoration mode of the LEDs
#
# @param { uint8_t mode }
# @see LIGHT_MODE_*
LIGHT_CMD_SET_MODE = 0xCF

# Light modes
# @see LIGHT_CMD_SET_MODE
LIGHT_MODE_STATIC = 0x0 # Stay on the selected colours, unchanging
LIGHT_MODE_BREATHE = 0x1 # Brightness pulses in the middle of the light that propagate out

# Light features
# @see LIGHT_CMD_SET_FEATURE
LIGHT_FEATURE_TRANSITION = 0x1 # Fade slowly to a colour instead of instantly changing
LIGHT_FEATURE_CALIBRATION = 0x2 # Correct the brightness of each colour to roughly match an SRGB display
LIGHT_FEATURE_CIE_BRIGHTNESS = 0x4 # Map the input brightness to apparent brightness

def controlWrite(handle, command, args=[]):
    '''
    Write a command byte argument bytes to a USB device
    '''
    handle.controlWrite(
        usb1.TYPE_VENDOR | usb1.RECIPIENT_DEVICE,
        usb1.REQUEST_SET_CONFIGURATION,
        0,
        0,
        [command] + args
    )

with usb1.USBContext() as context:
    handle = context.openByVendorIDAndProductID(
        VENDOR_ID,
        PRODUCT_ID,
        skip_on_error=True,
    )

    if handle is None:
    	print("Failed to find a device")
    	sys.exit(1)

    with handle.claimInterface(0):
        red = [255, 0, 0]
        off = [0, 0, 0]
        blue = [0, 0, 255]
        white = [255, 255, 255]

        controlWrite(handle, LIGHT_CMD_SET_FEATURE, [LIGHT_FEATURE_TRANSITION, 0])
        controlWrite(handle, LIGHT_CMD_SET_FEATURE, [LIGHT_FEATURE_CIE_BRIGHTNESS, 1])
        controlWrite(handle, LIGHT_CMD_SET_FEATURE, [LIGHT_FEATURE_CALIBRATION, 1])

        while True:

            for i in range(4):
                controlWrite(handle, LIGHT_CMD_SET_EACH, red + off + blue)
                time.sleep(0.05)
                controlWrite(handle, LIGHT_CMD_SET_EACH, off + off + off)
                time.sleep(0.05)

            controlWrite(handle, LIGHT_CMD_SET_EACH, red + off + blue)
            time.sleep(0.2)

            controlWrite(handle, LIGHT_CMD_SET_EACH, off * 3)
            time.sleep(0.1)

            for i in range(4):
                controlWrite(handle, LIGHT_CMD_SET_EACH, blue + off + red)
                time.sleep(0.05)
                controlWrite(handle, LIGHT_CMD_SET_EACH, off + off + off)
                time.sleep(0.05)

            controlWrite(handle, LIGHT_CMD_SET_EACH, blue + off + red)
            time.sleep(0.2)
